package de.ozgcloud.apilib.common.command.grpc;

import java.time.ZonedDateTime;

import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;

import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.OzgCloudCommandId;
import de.ozgcloud.apilib.common.command.OzgCloudCommandStatus;
import de.ozgcloud.apilib.common.datatypes.GenericId;
import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.common.datatype.StringBasedValue;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcCreateCommandRequest;

@Mapper(uses = { GrpcObjectMapper.class })
public interface CommandMapper {

	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeBodyObj", ignore = true)
	@Mapping(target = "mergeCallContext", ignore = true)
	@Mapping(target = "mergeRedirectRequest", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "removeBody", ignore = true)
	@Mapping(target = "callContext", ignore = true)
	@Mapping(target = "order", ignore = true)
	@Mapping(target = "orderStringBytes", ignore = true)
	@Mapping(target = "orderValue", ignore = true)
	@Mapping(target = "redirectRequest", ignore = true)
	@Mapping(target = "relationIdBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "vorgangIdBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "bodyBuilderList", ignore = true)
	@Mapping(target = "bodyList", ignore = true)
	@Mapping(target = "bodyOrBuilderList", ignore = true)

	@Mapping(target = "bodyObj", source = "bodyObject")
	@Mapping(target = "orderString", source = "order")
	GrpcCreateCommandRequest buildCreateCommandRequest(OzgCloudCommand commandToCreate);

	@Mapping(target = "bodyObject", source = "bodyObj")
	@Mapping(target = "order", source = "orderString")
	@Mapping(target = "relationVersion", ignore = true)
	@Mapping(target = "relationId", qualifiedByName = "mapRelationId")
	@Mapping(target = "body", ignore = true)
	@Mapping(target = "createdAt", expression = "java(mapToZonedDateTime(command.getCreatedAt()))")
	@Mapping(target = "finishedAt", expression = "java(mapToZonedDateTime(command.getFinishedAt()))")
	OzgCloudCommand fromGrpc(GrpcCommand command);

	default ZonedDateTime mapToZonedDateTime(String value) {
		return StringUtils.isEmpty(value) ? null : ZonedDateTime.parse(value);
	}

	@Named("mapRelationId")
	default StringBasedValue mapRelationId(String relationId) {
		return GenericId.from(relationId);
	}

	default OzgCloudVorgangId toOzgCloudVorgangId(String userId) {
		return OzgCloudVorgangId.from(userId);
	}

	default OzgCloudUserId toOzgCloudUserId(String userId) {
		return OzgCloudUserId.from(userId);
	}

	default OzgCloudCommandStatus toOzgCloudCommandStatus(String status) {
		return OzgCloudCommandStatus.from(status);
	}

	default OzgCloudCommandId toOzgCloudCommandId(String id) {
		return OzgCloudCommandId.from(id);
	}

	default String valueToString(StringBasedValue value) {
		return value.toString();
	}
}
